package util;

import com.meterware.httpunit.*;
import org.xml.sax.SAXException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.IOException;
import java.util.StringTokenizer;
import java.net.URLEncoder;

public class CddbLookUp {

    private static Log LOG = LogFactory.getLog(CddbLookUp.class);

    public static void lookup(SongInfo songInfo) throws IOException, SAXException, IdaNotFoundException {
        LOG.info("looking up data: " + songInfo);
        String artist = songInfo.getArtist();
        String title = songInfo.getTitle();
        if (artist == null || title == null) {
            throw new RuntimeException("unable to fetch ID3 info - title and artist are required");
        }
        WebConversation wc = new WebConversation();
        WebRequest req = new GetMethodWebRequest("http://www.gracenote.com/music/index_old.html");
        WebResponse resp = wc.getResponse(req);
        WebForm searchForm = resp.getFormWithName("search");
        searchForm.setParameter("q", URLEncoder.encode(artist + " " + title, "UTF-8"));
        WebResponse webResponse = searchForm.submit();

        LOG.debug("submitted initial search");

        if (webResponse.getTitle().toUpperCase().indexOf("SEARCH") != -1) {

            WebLink[] links = webResponse.getLinks();
            boolean found = false;

            LOG.debug("looking for artist: " + artist);
            for (WebLink link : links) {
                String text = link.getText();

                LOG.debug("line: " + text);
                if (text.toUpperCase().indexOf(artist.toUpperCase()) != -1) {
                    webResponse = link.click();
                    found = true;
                    break;
                }
            }
            if (!found) {
                throw new IdaNotFoundException("no IDA tag found for artist: " + songInfo);
            }

            LOG.debug("clicked through");
        } else {

            LOG.debug("skipped clicked through");
        }

        HTMLElement[] albumTd = webResponse.getElementsWithAttribute("class", "cddb_returns");

        String[] strings = albumTd[0].getText().split("\n");
        for (String text : strings) {

            LOG.debug("ele: " + text);
            if ((text.toUpperCase().indexOf(artist.toUpperCase()) != -1) && songInfo.getAlbum() == null) {
                String album = text.substring(text.indexOf("/") + 1).trim();

                LOG.debug("setting album: " + album);
                songInfo.setAlbum(album);
            } else if (text.indexOf("Label") != -1) {
                String str = "Label: ";
                String lable = text.substring(text.indexOf(str) + str.length()).trim();

                LOG.debug("setting label: " + lable);
                songInfo.setLable(lable);
            } else if (text.indexOf("Year") != -1) {
                String str = "Year: ";
                String year = text.substring(text.indexOf(str) + str.length()).trim();

                LOG.debug("setting year: " + year);
                songInfo.setYear(year);
            } else if (text.toUpperCase().indexOf(title.toUpperCase()) != -1) {
                text = text.replaceAll("\\|", "");
                StringTokenizer track = new StringTokenizer(text, ".");
                if (track.countTokens() != 2) {

                    LOG.debug("ignoring: " + text);
                    continue;
                }
                String trackNumber = track.nextToken();

                LOG.debug("setting track: " + trackNumber);
                songInfo.setTrackNumber(trackNumber.trim());
            } else if (text.indexOf("Digital") != -1) {
                String genre = text.trim().substring(0, text.indexOf("Digital")).trim();

                LOG.debug("setting genre: " + genre);
                songInfo.setGenre(genre);
            }
        }
        LOG.info("complete");
    }

}
